==> klibido-0.2.1.tar.gz: klibido-0.2.1/src/binheader.cpp <==
/***************************************************************************
 *   Copyright (C) 2004 by Alessandro Bonometti                            *
 *   bauno@bauniga.baita                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "binheader.h"

BinHeader::BinHeader() {
    size=0;
    lines=0;
	status=BinHeader::bh_new;
}

//Puts the size and the data of string s into array p;
//moves p to the next free element;
//returns p

uchar* BinHeader::insert(QString s, uchar* p) {
    int strlen=s.length();
    int suint = sizeof(strlen);
    memcpy(p, &strlen, suint);

    p+=suint;
    memcpy(p, (const char *)s, strlen);
    p+=strlen;
    return p;



}


// Retrieves the string at current position in array i into the QString s
// updates and returns i;

uchar* BinHeader::retrieve(uchar* i,QString &s) {

    int strlen;
    int ssize=sizeof(strlen);

    char *temp;
    memcpy(&strlen, i, ssize);

    i+=ssize;
    temp=new char[strlen+1];
    memcpy(temp, i, strlen);
    temp[strlen]='\0';
    s=temp;
    delete temp;
    i+=strlen;
    return i;



}

// Marshall the binHeader

uchar *BinHeader::data() {



    uchar *p=new uchar[getRecordSize()];
    uchar *i=&p[0];
    i=insert(subj, i);
    i=insert(from,i);
    i=insert(date,i);

    //copy fixed length fields...no big deal
    int szInt=sizeof(uint);

    //numparts field

    memcpy(i, &numParts, szInt);
    i+=szInt;

    // missingparts field;
    memcpy(i, &missingParts, szInt);
    ;
    i+=szInt;

    //status field

    memcpy(i, &status, szInt);
    i+=szInt;

    //lines field
    memcpy(i, &lines, szInt);
    i+=szInt;

    //size field
    memcpy(i, &size, szInt);
    i+=szInt;


    //how many items in the map?
    int partNumItems=partNum.count();
    memcpy(i, &partNumItems , szInt);
    i+=szInt;

    //Couples serverid-article number (at least 1)
    int part, serverNumItems, serverId, artNum;
    for (pnmit=partNum.begin(); pnmit != partNum.end(); ++pnmit) {
        //save part number - number of items in server nummap, then the couples
        // serverid-artNum
        part=pnmit.key();
        memcpy(i, &part, szInt);
        i+=szInt;
        serverNumItems=pnmit.data().count();
        memcpy(i, &serverNumItems, szInt);
        i+=szInt;
        for (snmit=pnmit.data().begin(); snmit != pnmit.data().end(); ++snmit) {
            serverId=snmit.key();
            artNum=snmit.data();
            memcpy(i, &serverId, szInt);
            i+=szInt;
            memcpy(i, &artNum, szInt);
            i+=szInt;
        }



    }
    // save servernummap count
    int serverPartItems=serverPart.count();

    memcpy(i, &serverPartItems, szInt);
    i+=szInt;
    for (spmit=serverPart.begin() ; spmit != serverPart.end(); ++spmit) {
        serverId=spmit.key();
        artNum=spmit.data();
        memcpy(i, &serverId, szInt);
        i+=szInt;
        memcpy(i, &artNum, szInt);
        i+=szInt;
    }
	
	//Using serverPartitems for partSize
	serverPartItems=partSize.count();
	
	memcpy(i, &serverPartItems, szInt);
	i+=szInt;
	QMap<uint, uint>::iterator psit;
	//Using serverId, artNum to limit autovariables allocation
	for (psit=partSize.begin(); psit !=partSize.end(); ++psit) {
		serverId=psit.key();
		artNum=psit.data();
		memcpy(i, &serverId, szInt);
		i+=szInt;
		memcpy(i, &artNum, szInt);
		i+=szInt;
		
	}
			







    return p;




}

//create binheader, unmarshalling DB data

BinHeader::BinHeader(uchar *p, int version) {


    uchar *pi =&p[0];


    // Variable-lenght fields

    pi=retrieve(pi,subj);
    pi=retrieve(pi,from);
    pi=retrieve(pi,date);


    // Fixed-size fields:
    int szInt=sizeof(int);

    memcpy(&numParts, pi, szInt);
    pi+=szInt;
    memcpy(&missingParts, pi, szInt);
    pi+=szInt;
    memcpy(&status, pi, szInt);
    pi+=szInt;
    //status
    memcpy(&lines, pi, szInt);
    pi+=szInt;
    memcpy(&size, pi, szInt);
    pi+=szInt;

    uint partNumItems;
    memcpy(&partNumItems, pi, szInt);
    pi+=szInt;


    uint  part, serverNumItems, serverId, artNum;
    // mapItems
    //     memcpy(&mapItems, pi, szInt);
    //     pi+=szInt;

    for (uint i = 0; i < partNumItems; i++) {
        // get the partid;
        memcpy(&part, pi, szInt);
        pi+=szInt;
        //get the number of items in the servernum map
        memcpy(&serverNumItems, pi, szInt);
        pi+=szInt;
        // now rebuild the servernum map
        // 		partNum.insert(part);
        for (uint j = 0; j < serverNumItems; j++) {
            memcpy(&serverId, pi, szInt);
            pi+=szInt;
            memcpy(&artNum, pi, szInt);
            pi+=szInt;
            partNum[part].insert(serverId, artNum);
        }
    }

    uint serverPartItems;
    memcpy(&serverPartItems, pi, szInt);
    pi+=szInt;
    for (uint i =0; i< serverPartItems; i++) {

        memcpy(&serverId, pi, szInt);
        pi+=szInt;
        memcpy(&artNum, pi, szInt);
        pi+=szInt;
        serverPart[serverId]=artNum;

    }
	
	//ok, if version >= 1 we have the size of the single parts to consider...
	//Using serverPartItems, serverId, artnum to limit autovariables allocation...


	if (version >= 1) {
// 		qDebug("Loading size into bh");
		memcpy(&serverPartItems, pi, szInt);
		pi+=szInt;
		for (uint i = 0; i < serverPartItems; i++) {
			memcpy(&serverId, pi, szInt);
			pi+=szInt;
			memcpy(&artNum, pi, szInt);
			pi+=szInt;
			partSize[serverId]=artNum;
		}
		
	}








}

//Updated to accomodate part sizes...

uint BinHeader::getRecordSize( ) {
    uint partNumSize=0;
    for (pnmit=partNum.begin(); pnmit != partNum.end(); ++pnmit) {
        partNumSize+=(pnmit.data().count()*2*sizeof(uint))+sizeof(uint);
        partNumSize+=sizeof(uint);
    }
    partNumSize+=sizeof(uint);


    return 10*sizeof(uint)+subj.length()+from.length()+date.length()+serverPart.count()*(2*sizeof(uint))+partNumSize \
			+ 2*(sizeof(uint)*partSize.count());
}

bool BinHeader::addPart(int part, Header *h) {

    if (!partNum.contains(part)) {
        partNum[part]=h->serverNum;
        // 		partNum[part]=h->serverNum;


        size+=h->m_bytes.toInt();
        lines+=h->m_lines.toInt();
        //         hi->setText(1,QString::number(size));
        //         hi->setText(2,QString::number(lines));

        //         new KListViewItem(hi, h.m_subj,h.m_bytes, h.m_lines, h.m_from, h.m_mid);
        missingParts--;
        /*if (isCompleted())
            hi->setPixmap(0,BarIcon("button_ok", KIcon::SizeSmall));
        *///         else
        //             hi->setPixmap(0,BarIcon("button_cancel", KIcon::SizeSmall));
        return true;

    } else
        return false;



}
bool BinHeader::isCompleted() {
    return (missingParts == 0);
}

void BinHeader::printServerPart( ) {
    for (pnmit=partNum.begin() ; pnmit != partNum.end() ; ++pnmit) {
        qDebug("Part: %d", pnmit.key());
        for (snmit=pnmit.data().begin(); snmit != pnmit.data().end(); snmit++) {
            qDebug("\tserverid: %d; articlenum: %d", snmit.key(), snmit.data());
        }
    }
}

bool BinHeader::addPart( int part, Header *h, NntpHost *nh ) {
    if (!partNum.contains(part)) { //new part
        partNum[part].insert(nh->id, h->m_num);
        // 		partNum[part]=h->serverNum;
        if (serverPart.contains(nh->id))
            (serverPart[nh->id])++;
        else
            serverPart[nh->id]=1;
		
		//Ok, every time I add a part that was missing, the article becomes new
		status=BinHeader::bh_new;

		partSize[part]=h->m_bytes.toInt();
// 		qDebug("part Size: %d", h->m_bytes.toInt());
        size+=h->m_bytes.toInt();
        lines+=h->m_lines.toInt();
        //         hi->setText(1,QString::number(size));
        //         hi->setText(2,QString::number(lines));

        //         new KListViewItem(hi, h.m_subj,h.m_bytes, h.m_lines, h.m_from, h.m_mid);
		if (part==0) 
			numParts++;
		else missingParts--;
		
        /*if (isCompleted())
            hi->setPixmap(0,BarIcon("button_ok", KIcon::SizeSmall));
        *///         else
        //             hi->setPixmap(0,BarIcon("button_cancel", KIcon::SizeSmall));
        return true;

    } else if (!partNum[part].contains(nh->id))  {// already have this part, but on another server
        partNum[part].insert(nh->id, h->m_num);
        if (serverPart.contains(nh->id))
            (serverPart[nh->id])++;
        else
            serverPart[nh->id]=1;

        return true;
    } else if (partNum[part][nh->id]< h->m_num) { // replace part with new...
        partNum[part][nh->id]=h->m_num;
        return true;
    } else
        return false; //duplicate?

}




//This has to be modified to reflect the "newess" of the article in the return code...

BinHeader::Add_Code BinHeader::addPart( int part, Header *h, int hostId ) {
    if (!partNum.contains(part)) { //new part
        partNum[part].insert(hostId, h->m_num);
		partSize[part]=h->m_bytes.toInt();
// 		qDebug("Part size: %d", partSize[part]);
// 		qDebug("Part number: %d", h->m_num);
        // 		partNum[part]=h->serverNum;
        if (serverPart.contains(hostId))
            (serverPart[hostId])++;
        else
            serverPart[hostId]=1;
		
		//Ok, every time I add a part that was missing, the article becomes new
		

// 		qDebug("part Size: %d", h->m_bytes.toInt());
        size+=h->m_bytes.toInt();
        lines+=h->m_lines.toInt();
        //         hi->setText(1,QString::number(size));
        //         hi->setText(2,QString::number(lines));

        //         new KListViewItem(hi, h.m_subj,h.m_bytes, h.m_lines, h.m_from, h.m_mid);
		if (part==0) 
			numParts++;
		else missingParts--;
		
//        if (isCompleted())
//             hi->setPixmap(0,BarIcon("button_ok", KIcon::SizeSmall));
	//         else
        //             hi->setPixmap(0,BarIcon("button_cancel", KIcon::SizeSmall));
		if (status != bh_new) {
			status=bh_new;
			return Unread_Status;
		} else return New_Part;

    } else if (!partNum[part].contains(hostId))  {// already have this part, but on another server
        partNum[part].insert(hostId, h->m_num);
        if (serverPart.contains(hostId))
            (serverPart[hostId])++;
        else
            serverPart[hostId]=1;

        return New_Part;
    } else if (partNum[part][hostId]< h->m_num) { // replace part with new...
        partNum[part][hostId]=h->m_num;
        return New_Part;
    } else
        return Duplicate_Part; //duplicate?

}


BinHeader::Expire_Code BinHeader::expire( int hostId, int lw ) {
// 	qDebug("Expiring: %s", (const char *) subj);
    for (pnmit=partNum.begin(); pnmit != partNum.end() ; ++pnmit) { //for every part...
        if (pnmit.data().contains(hostId)) { //if the server has the part...
            //check the article number...
            if (pnmit.data()[hostId] < lw) { //if the artnum < lowWatermark...
                //remove the part...
// 				qDebug("Expiring part: ", pnmit.data()[hostId]);
                pnmit.data().remove(hostId);
				(serverPart[hostId])--;
				if (pnmit.data().isEmpty()) {
					partNum.remove(pnmit);
					partSize.remove(pnmit.key());
				}

            }
            // now if the serverNum is empty(i.e., article expired on all servers)
            // remove the part
            
        }
    }
    //now if the partList is empty, remove the post
	//To keep track of new articles I have to return 3 codes:
	// a) Don't delete (nothing changes)
	// b) Delete, but the article is not new
	// c) Delete a "new" article (unlikely, but...)
	
    if (partNum.isEmpty()) {
         //delete the post
// 		qDebug("Post deleted");
		
		if (status == bh_new)
			return Delete_Unread;
		else return Delete_Read;
    } else {
// 		qDebug("Elements still in partNum?");
        return No_Delete;//re-save the post...
	}








}

==> klibido-0.2.2.tar.gz: klibido-0.2.2/src/binheader.cpp <==
/***************************************************************************
 *   Copyright (C) 2004 by Alessandro Bonometti                            *
 *   bauno@bauniga.baita                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "binheader.h"

BinHeader::BinHeader() {
    size=0;
    lines=0;
	status=BinHeader::bh_new;
}

//Puts the size and the data of string s into array p;
//moves p to the next free element;
//returns p

uchar* BinHeader::insert(QString s, uchar* p) {
    int strlen=s.length();
    int suint = sizeof(strlen);
    memcpy(p, &strlen, suint);

    p+=suint;
    memcpy(p, (const char *)s, strlen);
    p+=strlen;
    return p;



}


// Retrieves the string at current position in array i into the QString s
// updates and returns i;

uchar* BinHeader::retrieve(uchar* i,QString &s) {

    int strlen;
    int ssize=sizeof(strlen);

    char *temp;
    memcpy(&strlen, i, ssize);

    i+=ssize;
    temp=new char[strlen+1];
    memcpy(temp, i, strlen);
    temp[strlen]='\0';
    s=temp;
    delete temp;
    i+=strlen;
    return i;



}

// Marshall the binHeader

uchar *BinHeader::data() {



    uchar *p=new uchar[getRecordSize()];
    uchar *i=&p[0];
    i=insert(subj, i);
    i=insert(from,i);
    i=insert(date,i);

    //copy fixed length fields...no big deal
    int szInt=sizeof(uint);

    //numparts field

    memcpy(i, &numParts, szInt);
    i+=szInt;

    // missingparts field;
    memcpy(i, &missingParts, szInt);
    ;
    i+=szInt;

    //status field

    memcpy(i, &status, szInt);
    i+=szInt;

    //lines field
    memcpy(i, &lines, szInt);
    i+=szInt;

    //size field
    memcpy(i, &size, szInt);
    i+=szInt;


    //how many items in the map?
    int partNumItems=partNum.count();
    memcpy(i, &partNumItems , szInt);
    i+=szInt;

    //Couples serverid-article number (at least 1)
    int part, serverNumItems, serverId, artNum;
    for (pnmit=partNum.begin(); pnmit != partNum.end(); ++pnmit) {
        //save part number - number of items in server nummap, then the couples
        // serverid-artNum
        part=pnmit.key();
        memcpy(i, &part, szInt);
        i+=szInt;
        serverNumItems=pnmit.data().count();
        memcpy(i, &serverNumItems, szInt);
        i+=szInt;
        for (snmit=pnmit.data().begin(); snmit != pnmit.data().end(); ++snmit) {
            serverId=snmit.key();
            artNum=snmit.data();
            memcpy(i, &serverId, szInt);
            i+=szInt;
            memcpy(i, &artNum, szInt);
            i+=szInt;
        }



    }
	//BEGIN NEW MID
	int partMidItems = partMid.count();
	memcpy(i, &partMidItems, szInt);
	i+=szInt;
	PartMid::Iterator partMidIt;
	
	for (partMidIt=partMid.begin(); partMidIt != partMid.end(); ++partMidIt) {
		part=partMidIt.key();
		memcpy(i, &part, szInt);
		i+=szInt;
		
		i=insert(partMidIt.data(), i);
		
	}
		
		
	
	//END NEW MID
    // save servernummap count
    int serverPartItems=serverPart.count();

    memcpy(i, &serverPartItems, szInt);
    i+=szInt;
    for (spmit=serverPart.begin() ; spmit != serverPart.end(); ++spmit) {
        serverId=spmit.key();
        artNum=spmit.data();
        memcpy(i, &serverId, szInt);
        i+=szInt;
        memcpy(i, &artNum, szInt);
        i+=szInt;
    }
	
	//Using serverPartitems for partSize
	serverPartItems=partSize.count();
	
	memcpy(i, &serverPartItems, szInt);
	i+=szInt;
	QMap<uint, uint>::iterator psit;
	//Using serverId, artNum to limit autovariables allocation
	for (psit=partSize.begin(); psit !=partSize.end(); ++psit) {
		serverId=psit.key();
		artNum=psit.data();
		memcpy(i, &serverId, szInt);
		i+=szInt;
		memcpy(i, &artNum, szInt);
		i+=szInt;
		
	}
			







    return p;




}

//create binheader, unmarshalling DB data

BinHeader::BinHeader(uchar *p, int version) {


    uchar *pi =&p[0];


    // Variable-lenght fields

    pi=retrieve(pi,subj);
    pi=retrieve(pi,from);
    pi=retrieve(pi,date);


    // Fixed-size fields:
    int szInt=sizeof(int);

    memcpy(&numParts, pi, szInt);
    pi+=szInt;
    memcpy(&missingParts, pi, szInt);
    pi+=szInt;
    memcpy(&status, pi, szInt);
    pi+=szInt;
    //status
    memcpy(&lines, pi, szInt);
    pi+=szInt;
    memcpy(&size, pi, szInt);
    pi+=szInt;

    uint partNumItems;
    memcpy(&partNumItems, pi, szInt);
    pi+=szInt;


    uint  part, serverNumItems, serverId, artNum;
    // mapItems
    //     memcpy(&mapItems, pi, szInt);
    //     pi+=szInt;

    for (uint i = 0; i < partNumItems; i++) {
        // get the partid;
        memcpy(&part, pi, szInt);
        pi+=szInt;
        //get the number of items in the servernum map
        memcpy(&serverNumItems, pi, szInt);
        pi+=szInt;
        // now rebuild the servernum map
        // 		partNum.insert(part);
        for (uint j = 0; j < serverNumItems; j++) {
            memcpy(&serverId, pi, szInt);
            pi+=szInt;
            memcpy(&artNum, pi, szInt);
            pi+=szInt;
            partNum[part].insert(serverId, artNum);
        }
    }
	//BEGIN partMid code
		
	if (version >= 2) {
		uint partMidItems;
		memcpy(&partMidItems, pi, szInt);
		pi+=szInt;
		QString mid;
		for (uint i = 0; i < partMidItems; i++) {
			memcpy(&part, pi, szInt);
			pi+=szInt;
			pi = retrieve(pi, mid);
			partMid[part]=mid;
			
		}
	
	}
	//END partMid code

    uint serverPartItems;
    memcpy(&serverPartItems, pi, szInt);
    pi+=szInt;
    for (uint i =0; i< serverPartItems; i++) {

        memcpy(&serverId, pi, szInt);
        pi+=szInt;
        memcpy(&artNum, pi, szInt);
        pi+=szInt;
        serverPart[serverId]=artNum;

    }
	
	//ok, if version >= 1 we have the size of the single parts to consider...
	//Using serverPartItems, serverId, artnum to limit autovariables allocation...


	if (version >= 1) {
// 		qDebug("Loading size into bh");
		memcpy(&serverPartItems, pi, szInt);
		pi+=szInt;
		for (uint i = 0; i < serverPartItems; i++) {
			memcpy(&serverId, pi, szInt);
			pi+=szInt;
			memcpy(&artNum, pi, szInt);
			pi+=szInt;
			partSize[serverId]=artNum;
		}
		
	}








}

//Updated to accomodate part sizes...

uint BinHeader::getRecordSize( ) {
    uint partNumSize=0;
    for (pnmit=partNum.begin(); pnmit != partNum.end(); ++pnmit) {
        partNumSize+=(pnmit.data().count()*2*sizeof(uint))+sizeof(uint);
        partNumSize+=sizeof(uint);
    }
    partNumSize+=sizeof(uint);

	/* For every part-mid couple, we have:
		- A int which holds the number of parts
		- A int which holds the length of the string (mid)
		- the length of the string.
		- The first two depends on the number of parts only (i.e.: they can be calculated 
		  with 2*sizeof(int)*partMid.count() + sizeof(int); (including the int that holds the number of
		  parts
		- To know the latter, we have to scan ALL the parts :-\
		
	*/
	PartMid::Iterator partMidIt;
	uint partMidSize = 2*sizeof(int)*partMid.count()+sizeof(int);
	for (partMidIt = partMid.begin(); partMidIt != partMid.end(); ++partMidIt) {
		
		
		partMidSize+=partMidIt.data().length();
		
	}
	
	
			
			
    return 10*sizeof(uint)+subj.length()+from.length()+date.length()+serverPart.count()*(2*sizeof(uint))+partNumSize \
			+ 2*(sizeof(uint)*partSize.count()) + partMidSize;
}

bool BinHeader::addPart(int part, Header *h) {

    if (!partNum.contains(part)) {
        partNum[part]=h->serverNum;
        // 		partNum[part]=h->serverNum;


        size+=h->m_bytes.toInt();
        lines+=h->m_lines.toInt();
        //         hi->setText(1,QString::number(size));
        //         hi->setText(2,QString::number(lines));

        //         new KListViewItem(hi, h.m_subj,h.m_bytes, h.m_lines, h.m_from, h.m_mid);
        missingParts--;
        /*if (isCompleted())
            hi->setPixmap(0,BarIcon("button_ok", KIcon::SizeSmall));
        *///         else
        //             hi->setPixmap(0,BarIcon("button_cancel", KIcon::SizeSmall));
        return true;

    } else
        return false;



}
bool BinHeader::isCompleted() {
    return (missingParts == 0);
}

void BinHeader::printServerPart( ) {
    for (pnmit=partNum.begin() ; pnmit != partNum.end() ; ++pnmit) {
        qDebug("Part: %d", pnmit.key());
        for (snmit=pnmit.data().begin(); snmit != pnmit.data().end(); snmit++) {
            qDebug("\tserverid: %d; articlenum: %d", snmit.key(), snmit.data());
        }
    }
}

bool BinHeader::addPart( int part, Header *h, NntpHost *nh ) {
    if (!partNum.contains(part)) { //new part
        partNum[part].insert(nh->id, h->m_num);
        // 		partNum[part]=h->serverNum;
        if (serverPart.contains(nh->id))
            (serverPart[nh->id])++;
        else
            serverPart[nh->id]=1;
		
		//Ok, every time I add a part that was missing, the article becomes new
		status=BinHeader::bh_new;

		partSize[part]=h->m_bytes.toInt();
// 		qDebug("part Size: %d", h->m_bytes.toInt());
        size+=h->m_bytes.toInt();
        lines+=h->m_lines.toInt();
        //         hi->setText(1,QString::number(size));
        //         hi->setText(2,QString::number(lines));

        //         new KListViewItem(hi, h.m_subj,h.m_bytes, h.m_lines, h.m_from, h.m_mid);
		if (part==0) 
			numParts++;
		else missingParts--;
		
        /*if (isCompleted())
            hi->setPixmap(0,BarIcon("button_ok", KIcon::SizeSmall));
        *///         else
        //             hi->setPixmap(0,BarIcon("button_cancel", KIcon::SizeSmall));
        return true;

    } else if (!partNum[part].contains(nh->id))  {// already have this part, but on another server
        partNum[part].insert(nh->id, h->m_num);
        if (serverPart.contains(nh->id))
            (serverPart[nh->id])++;
        else
            serverPart[nh->id]=1;

        return true;
    } else if (partNum[part][nh->id]< h->m_num) { // replace part with new...
        partNum[part][nh->id]=h->m_num;
        return true;
    } else
        return false; //duplicate?

}





void BinHeader::addNzbPart(int part, int bytes, QString mid )
{
	partSize[part]=bytes;
	partMid[part]=mid;
	size+=bytes;
	missingParts--;
	
	
}

BinHeader::Add_Code BinHeader::addPart( int part, Header *h, int hostId ) {
    if (!partNum.contains(part)) { //new part
        partNum[part].insert(hostId, h->m_num);
		partSize[part]=h->m_bytes.toInt();
// 		qDebug("Part size: %d", partSize[part]);
// 		qDebug("Part number: %d", h->m_num);
        // 		partNum[part]=h->serverNum;
        if (serverPart.contains(hostId))
            (serverPart[hostId])++;
        else
            serverPart[hostId]=1;
		
		//Ok, every time I add a part that was missing, the article becomes new
		

// 		qDebug("part Size: %d", h->m_bytes.toInt());
        size+=h->m_bytes.toInt();
        lines+=h->m_lines.toInt();
        //         hi->setText(1,QString::number(size));
        //         hi->setText(2,QString::number(lines));

        //         new KListViewItem(hi, h.m_subj,h.m_bytes, h.m_lines, h.m_from, h.m_mid);
		if (part==0) 
			numParts++;
		else missingParts--;
		
//        if (isCompleted())
//             hi->setPixmap(0,BarIcon("button_ok", KIcon::SizeSmall));
	//         else
        //             hi->setPixmap(0,BarIcon("button_cancel", KIcon::SizeSmall));
		if (status != bh_new) {
			status=bh_new;
			return Unread_Status;
		} else return New_Part;

    } else if (!partNum[part].contains(hostId))  {// already have this part, but on another server
        partNum[part].insert(hostId, h->m_num);
        if (serverPart.contains(hostId))
            (serverPart[hostId])++;
        else
            serverPart[hostId]=1;

        return New_Part;
    } else if (partNum[part][hostId]< h->m_num) { // replace part with new...
        partNum[part][hostId]=h->m_num;
        return New_Part;
    } else
        return Duplicate_Part; //duplicate?

}


BinHeader::Expire_Code BinHeader::expire( int hostId, int lw ) {
// 	qDebug("Expiring: %s", (const char *) subj);
    for (pnmit=partNum.begin(); pnmit != partNum.end() ; ++pnmit) { //for every part...
        if (pnmit.data().contains(hostId)) { //if the server has the part...
            //check the article number...
            if (pnmit.data()[hostId] < lw) { //if the artnum < lowWatermark...
                //remove the part...
// 				qDebug("Expiring part: ", pnmit.data()[hostId]);
                pnmit.data().remove(hostId);
				(serverPart[hostId])--;
				if (pnmit.data().isEmpty()) {
					partNum.remove(pnmit);
					partSize.remove(pnmit.key());
				}

            }
            // now if the serverNum is empty(i.e., article expired on all servers)
            // remove the part
            
        }
    }
    //now if the partList is empty, remove the post
	//To keep track of new articles I have to return 3 codes:
	// a) Don't delete (nothing changes)
	// b) Delete, but the article is not new
	// c) Delete a "new" article (unlikely, but...)
	
    if (partNum.isEmpty()) {
         //delete the post
// 		qDebug("Post deleted");
		
		if (status == bh_new)
			return Delete_Unread;
		else return Delete_Read;
    } else {
// 		qDebug("Elements still in partNum?");
        return No_Delete;//re-save the post...
	}








}

void BinHeader::addHost( int part, int hostId )
{
	//Fake hosts, for chooseserver et al.
	partNum[part].insert(hostId, 0);
}

/*
BinHeader::~BinHeader( )
{
	kdDebug() << "Deleting BinHeader\n";
}
*/
